#!/usr/bin/perl -w

=head1 NAME

 mpbias.pl - A wrapper script for pbias executable

=head1 SYNOPSIS

 mpbias.pl <input FASTA file> <output file> [optional arguments]

=head1 ARGUMENTS

 <input FASTA file> : Input FASTA file.
 <output file>      : Masked FASTA file.
 [optional arguments] : Optional arguments used by pbias executable.
   -W:# - scanning window size
   -E:# - fragments closer than this number of positions will be merged
          (4 by default)
   -S:# - shuffle the original sequence # times to generate random sequences
   -G:# - use a.a. frequencies to generate # random sequences
   -T   - use SPROT frequencies (default)
   -X   - use PDB frequencies
   -I   - estimate frequencies from input sequence
   -F   - output PSEG-like masked sequence in FASTA format
          (segments with p-value < min. p-value are masked by X's)
   -L   - mask segments using lower-case letters
   -N   - do not merge overlapping/adjacent scanning windows
   -K   - estimate linkage distance from expected number of occurrences
   -A:string - amino acid sub-alphabet (default: GPSTDNH)
   -P:# - min. p-value for merging adjacent scanning windows (0.05 by default)
   -M:# - min. p-value for masking FASTA output (0.05 by default)

=head1 NOTE

 - Input FASTA file should contain ONLY ONE SEQUENCE.
 - mpbias.pl, pbias executable, and input FASTA file should be
   IN THE SAME DIRECTORY.

=head1 DESCRIPTION

 Compositionally biased segments in the input FASTA file are masked
 across all 20 amino acids, and the masked sequence is written
 into the output file.

=head1 LAST UPDATED

 12/21/2005

=head1 SEE ALSO

 Documentation for pbias executable.

=cut

use strict;
use warnings;

#----------------------------------------
# Arguments.
#----------------------------------------
my $usage = 'Usage: mpbias.pl <input FASTA file> <output file> [optional arguments]';

unless(defined(@ARGV) && scalar(@ARGV)>=2)
 {
   print("$usage\n");
   exit;
 }

my $file_in_fasta = shift(@ARGV);
my $file_out      = shift(@ARGV);
my $optional_args = join(' ', @ARGV);

#----------------------------------------------------------------------
# [Step 1]
# Run pbias executable to generate 20 .fas and 20 .seg files.
# -k switch for estimating linkage distance, -f switch generates .fas file.
# -m:0.0025 sets masking p-value to 0.0025 to account for multiple testing (20 runs)
#----------------------------------------------------------------------
my @AAs = qw(A C D E F G H I K L M N P Q R S T V W Y);

foreach my $AA (@AAs)
 {
   my $file_out_pbias = 'pbias.temp.' . $AA;
   my $args = "$file_in_fasta $file_out_pbias -a:$AA -k -f -m:0.0025 $optional_args";
   system("./pbias $args");
 }

#----------------------------------------------------------------------
# [Step 2]
# Save masked sequence strings across 20 .fas files
# and delete .fas and .seg files.
#----------------------------------------------------------------------
my $header;
my %seq_strings = ();

foreach my $AA (@AAs)
 {
   my $file_in_fas = 'pbias.temp.' . $AA . '.fas';
   open(IN, $file_in_fas) || die("Cannot open $file_in_fas");
   while(<IN>)
    {
      if(/^>(.+)$/)
       {
	 $header = $1;
       }
      else
       {
	 $seq_strings{$AA} .= $_;
       }
    }
   close(IN);
   $seq_strings{$AA} =~ s/\s//g;
 }

foreach my $AA (@AAs)
 {
   my $file_in_fas = 'pbias.temp.' . $AA . '.fas';
   my $file_in_seg = 'pbias.temp.' . $AA . '.seg';
   unlink($file_in_fas, $file_in_seg);
 }

#----------------------------------------------------------------------
# [Step 3]
# Get the 'consensus' across 20 .fas files. The consensus is:
# - X if masking occurred for any of the 20 AAs,
# - original AA otherwise.
#----------------------------------------------------------------------
my $string_leng = length($seq_strings{$AAs[0]});

my $masked_string = '';

for(my $i=0; $i<=$string_leng-1; $i++)
 {
   my @curr_chars = ();

   foreach my $AA (@AAs)
    {
      my $curr_char = substr($seq_strings{$AA}, $i, 1);
      push(@curr_chars, $curr_char);
    }

   my $j=0;

   foreach my $curr_char (@curr_chars)
    {
      $j++;

      if($curr_char eq 'X')
       {
	 $masked_string .= $curr_char;
	 last;
       }

      if($j==scalar(@curr_chars))
       {
	 $masked_string .= $curr_char;
       }
    }
 }

open(OUT, ">$file_out") || die("Cannot open $file_out:$!");
print OUT (">$header\n");
print OUT ("$masked_string\n");
close(OUT);
